SENTRY_DSN = ''
MMSTATS_BASE_DIR = ''

SERVICES = {
    'email_sender': {
        'queues': {
            'email_sender__register': {
                'durable': True,
                'want_reply': False,
                'x-message-ttl': 600 * 1000,
            },
            'email_sender__password_reset': {
                'durable': True,
                'want_reply': False,
                'x-message-ttl': 600 * 1000,
            },
            'email_sender__pkgups_notif': {
                'durable': True,
                'want_reply': False,
                'x-message-ttl': 3600 * 1000,
            },
            'email_sender__contact_form': {
                'durable': True,
                'want_reply': False,
                'x-message-ttl': 36000 * 1000,
            },
        },
    },

    'nut_indexing': {
        'queues': {
            'nut_indexing': {
                'durable': False,
                'want_reply': True,
                'x-message-ttl': 30 * 1000,
                'default_timeout': 5.0,
            },
        },
    },

###

    'example': {
        'queues': {
            'example__q1': {
                'durable': False,
                'want_reply': True,
            },
            'example__q2': {
                'durable': False,
                'want_reply': True,
            },
        },
    },
}

