
class Serializer(object):

    def from_bytes(self, bs):
        '''Creates a rich object from the raw string.
        '''
        raise NotImplementedError()

    def to_bytes(self, p):
        '''Serializes a rich object to a raw string.
        '''
        raise NotImplementedError()


class ResponseProducer(object):

    def __call__(self, request_message):
        raise NotImplementedError()


